
/* A5SES is released under the zlib license.
 * See LICENSE.txt for more info.
 */

/// @file Color.hpp
/// @ingroup Color
/// Contains color related classes

#ifndef A5SES_COLOR_HPP_INCLUDED
#define A5SES_COLOR_HPP_INCLUDED

#include "common.hpp"

namespace a5
{

/// @ingroup Color
/// Stores Colors in the RGB format
struct RGB
{
	/// Red color component
	int r;

	/// Green color component
	int g;

	/// Blue color component
	int b;

	/// Creates an RGB object with the specified values
	RGB(int r_, int g_, int b_) : r(r_), g(g_), b(b_) {}
};

/// @ingroup Color
/// Stores Colors in the RGBA format
struct RGBA
{
	/// Red color component
	int r;

	/// Green color component
	int g;

	/// Blue color component
	int b;

	/// Alpha (transparency) component
	int a;

	/// Creates an RGBA object with the specified values
	RGBA(int r_, int g_, int b_, int a_) : r(r_), g(g_), b(b_), a(a_) {}
};

/// @ingroup Color
/// Stores and converts colors
class Color
{
	protected:
		/// Allegro color
		ALLEGRO_COLOR color;

	public:
		/// Creates a color
		Color() { }

		Color(int c) : color(al_map_rgb((c << 16) & 0xFF, (c << 8) & 0xFF, c & 0xFF)) { }
		Color(ALLEGRO_COLOR c) : color(c) { }
		Color(RGB c) : color(al_map_rgb(c.r, c.g, c.b)) { }
		Color(RGBA c) : color(al_map_rgba(c.r, c.g, c.b, c.a)) { }

		/// Returns the Allegro color structure
		operator ALLEGRO_COLOR()
		{
			return this->color;
		}
};

}

#endif // A5SES_COLOR_HPP_INCLUDED
