
/* A5SES is released under the zlib license.
 * See LICENSE.txt for more info.
 */

/// @file Timer.hpp
/// @ingroup Timer
/// Contains timing related classes and functions

#ifndef A5SES_TIMER_HPP_INCLUDED
#define A5SES_TIMER_HPP_INCLUDED

#include "common.hpp"
#include "Event.hpp"

namespace a5
{

class Timer : public Event_Source
{
	protected:
		ALLEGRO_TIMER *timer;

	public:
		/// @ingroup Event Timer
		/// Timer event object
		class Event : public a5::Event
		{
			public:
				enum Sub
				{
					Tick = 0x00000001,
					Any = 0xFFFFFFFF
				};

				static const int TypeID = 0x00000010;

				/// Creates a Timer event
				Event(const Event_Source *source_, int subtype_)
				{
					this->source = source_;
					this->subtype = subtype_;
				}

				virtual int Type()
				{
					return TypeID;
				}

		};

		Timer(float hz)
		{
			_init();
			this->timer = al_install_timer(1.f / hz);
			this->event_source = reinterpret_cast<ALLEGRO_EVENT_SOURCE *>(this->timer);
			//this->event_source = al_get_timer_event_source(this->timer);
		}

		void Start()
		{
			al_start_timer(this->timer);
		}

		void Stop()
		{
			al_stop_timer(this->timer);
		}

		virtual std::auto_ptr<a5::Event> Handle(ALLEGRO_EVENT *raw_event) const
		{
			std::auto_ptr<a5::Event> event(0);

			if (raw_event->any.source == this->event_source)
			{
				event = std::auto_ptr<a5::Event>(new Event(this, Event::Tick));
			}

			return event;
		}
};

/// @ingroup Timer
/// Pauses the thread for the specified number of seconds.
inline void Rest(float seconds)
{
	al_rest(seconds);
}

/// @ingroup Timer
/// Returns the number of seconds since Allegro was initialized
inline double Time()
{
	_init();
	return al_current_time();
}

}

#endif // A5SES_TIMER_HPP_INCLUDED
